@@ -3,9 +3,6 @@
 # symbols, and then we prune some below if they aren't available on this
 # system.
 __all__ = [
-    "Result",
-    "Value",
-    "Error",
     "cancel_shielded_checkpoint",
     "Abort",
     "wait_task_rescheduled",
@@ -23,14 +20,16 @@ __all__ = [
     "add_instrument",
     "current_clock",
     "current_statistics",
-    "wait_writable",
-    "wait_readable",
     "ParkingLot",
     "UnboundedQueue",
     "RunLocal",
     "RunVar",
+    "wait_writable",
+    "wait_readable",
+    "notify_fd_close",
     "wait_socket_readable",
     "wait_socket_writable",
+    "notify_socket_close",
     "TrioToken",
     "current_trio_token",
     # kqueue symbols
