@@ -7,6 +7,7 @@ import tempfile
 
 from .._core.tests.tutil import need_ipv6
 from .. import _core
+from .. import _socket as _tsocket
 from .. import socket as tsocket
 from .._socket import _NUMERIC_ONLY, _try_sync
 from ..testing import assert_checkpoints, wait_all_tasks_blocked
@@ -308,7 +309,7 @@ async def test_SocketType_basics():
     # type family proto
     stdlib_sock = stdlib_socket.socket()
     sock = tsocket.from_stdlib_socket(stdlib_sock)
-    assert sock.type == stdlib_sock.type
+    assert sock.type == _tsocket.real_socket_type(stdlib_sock.type)
     assert sock.family == stdlib_sock.family
     assert sock.proto == stdlib_sock.proto
     sock.close()
@@ -838,3 +839,33 @@ async def test_unix_domain_socket():
     except FileNotFoundError:
         # MacOS doesn't support abstract filenames with the leading NUL byte
         pass
+
+
+async def test_interrupted_by_close():
+    a_stdlib, b_stdlib = stdlib_socket.socketpair()
+    with a_stdlib, b_stdlib:
+        a_stdlib.setblocking(False)
+
+        data = b"x" * 99999
+
+        try:
+            while True:
+                a_stdlib.send(data)
+        except BlockingIOError:
+            pass
+
+        a = tsocket.from_stdlib_socket(a_stdlib)
+
+        async def sender():
+            with pytest.raises(_core.ClosedResourceError):
+                await a.send(data)
+
+        async def receiver():
+            with pytest.raises(_core.ClosedResourceError):
+                await a.recv(1)
+
+        async with _core.open_nursery() as nursery:
+            nursery.start_soon(sender)
+            nursery.start_soon(receiver)
+            await wait_all_tasks_blocked()
+            a.close()
