@@ -5,7 +5,8 @@ import weakref
 from ..testing import wait_all_tasks_blocked, assert_checkpoints
 
 from .. import _core
-from .._timeouts import sleep_forever
+from .. import _timeouts
+from .._timeouts import sleep_forever, move_on_after
 from .._sync import *
 
 
@@ -378,8 +379,6 @@ async def test_Queue():
         Queue(1.0)
     with pytest.raises(ValueError):
         Queue(-1)
-    with pytest.raises(ValueError):
-        Queue(0)
 
     q = Queue(2)
     repr(q)  # smoke test
@@ -410,6 +409,14 @@ async def test_Queue():
     assert q.empty()
 
 
+async def test_553(autojump_clock):
+    q = Queue(1)
+    with move_on_after(10) as timeout_scope:
+        await q.get()
+    assert timeout_scope.cancelled_caught
+    await q.put("Test for PR #553")
+
+
 async def test_Queue_iter():
     q = Queue(1)
 
@@ -512,6 +519,29 @@ async def test_Queue_fairness():
         assert (await q.get()) == 2
 
 
+async def test_Queue_unbuffered():
+    q = Queue(0)
+    assert q.capacity == 0
+    assert q.qsize() == 0
+    assert q.empty()
+    assert q.full()
+    with pytest.raises(_core.WouldBlock):
+        q.get_nowait()
+    with pytest.raises(_core.WouldBlock):
+        q.put_nowait(1)
+
+    async def do_put(q, v):
+        with assert_checkpoints():
+            await q.put(v)
+
+    async with _core.open_nursery() as nursery:
+        nursery.start_soon(do_put, q, 1)
+        with assert_checkpoints():
+            assert await q.get() == 1
+    with pytest.raises(_core.WouldBlock):
+        q.get_nowait()
+
+
 # Two ways of implementing a Lock in terms of a Queue. Used to let us put the
 # Queue through the generic lock tests.
 
@@ -551,6 +581,34 @@ class QueueLock2:
         self.q.put_nowait(None)
 
 
+@async_cm
+class QueueLock3:
+    def __init__(self):
+        self.q = Queue(0)
+        # self.acquired is true when one task acquires the lock and
+        # only becomes false when it's released and no tasks are
+        # waiting to acquire.
+        self.acquired = False
+
+    def acquire_nowait(self):
+        assert not self.acquired
+        self.acquired = True
+
+    async def acquire(self):
+        if self.acquired:
+            await self.q.put(None)
+        else:
+            self.acquired = True
+            await _core.checkpoint()
+
+    def release(self):
+        try:
+            self.q.get_nowait()
+        except _core.WouldBlock:
+            assert self.acquired
+            self.acquired = False
+
+
 lock_factories = [
     lambda: CapacityLimiter(1),
     lambda: Semaphore(1),
@@ -559,6 +617,7 @@ lock_factories = [
     lambda: QueueLock1(10),
     lambda: QueueLock1(1),
     QueueLock2,
+    QueueLock3,
 ]
 lock_factory_names = [
     "CapacityLimiter(1)",
@@ -568,6 +627,7 @@ lock_factory_names = [
     "QueueLock1(10)",
     "QueueLock1(1)",
     "QueueLock2",
+    "QueueLock3",
 ]
 
 generic_lock_test = pytest.mark.parametrize(
